@@ -6,61 +6,65 @@ from datetime import datetime
 import traceback
 import pandas as pd
 from s3fs import S3FileSystem
+from ..lib.exceptions import NoDataFound
 
 from dateutil import parser as dateparser
 from datetime import datetime, date, timedelta
 
+
 class DataSource(object):
     """
     Class for specifying a dataset. This is a base class
     meant to be derived and implemented.
     """
+
     def __init__(self, params, *args, **kwargs):
 
-        self._type  = params.get('type', 'unknown')
+        self._type = params.get("type", "unknown")
         """
         Type of the dataset.
         """
 
-        self.name = params['name']
+        self.name = params["name"]
         """
         Unique name of the dataset, e.g., Events
         """
 
-        self.description = params.get('description','')
+        self.description = params.get("description", "")
         """
         str: Description
         Example: "Daily events for Alpha Sensor"
         """
 
-        self.version = params.get('version','v1')
+        self.version = params.get("version", "v1")
         """
         str: Version
         Example: "v1"
         """
 
-        self.resolve = kwargs.get('resolve',
-                                  params.get('resolve',{}))
+        self.resolve = kwargs.get("resolve", params.get("resolve", {}))
         """
         Parameters to resolve paths
         """
 
         # Backward compatability
-        if 'paths' not in params:
-            params['paths'] = []
-            for name in ['local', 'test', 'backup']:
+        if "paths" not in params:
+            params["paths"] = []
+            for name in ["local", "test", "backup"]:
                 if name in params:
-                    params['paths'].append({
-                        'name': name,
-                        'nature': 's3' if name == 'backup' else 'local',
-                        'path': params[name]
-                    })
+                    params["paths"].append(
+                        {
+                            "name": name,
+                            "nature": "s3" if name == "backup" else "local",
+                            "path": params[name],
+                        }
+                    )
 
         self.validate(params)
 
         self._type = "File"
 
-        self.isfile = params.get('isfile', False)
+        self.isfile = params.get("isfile", False)
         """
         Specifies whether the dataset is simple (a file for each
         run) or more complex directory hierarchy.
@@ -74,9 +78,9 @@ class DataSource(object):
                 ...
 
         """
-        self.issortable = params.get('issortable', True)
+        self.issortable = params.get("issortable", True)
 
-        self.subsets = params.get('subsets',[])
+        self.subsets = params.get("subsets", [])
         """
         A dataset oftentimes has multiple components. Define
         them here. We can give a name to each.
@@ -94,8 +98,7 @@ class DataSource(object):
           }
         """
 
-
-        self.paths = params.get('paths',{})
+        self.paths = params.get("paths", {})
         """
         dict: Paths to be resolved
         Example::
@@ -124,7 +127,7 @@ class DataSource(object):
         """
         return self.paths
 
-    def get_path_by_name(self,name, full=False, resolve=None):
+    def get_path_by_name(self, name, full=False, resolve=None):
         """
         Return the path definition for a path with a given name
         """
@@ -134,7 +137,7 @@ class DataSource(object):
 
         selected = None
         for p in self.paths:
-            if p['name'] == name:
+            if p["name"] == name:
                 selected = p
                 break
         if selected is None:
@@ -143,12 +146,14 @@ class DataSource(object):
         if full:
             return selected
 
-        path = selected['path']
+        path = selected["path"]
 
         try:
-            if ((resolve is not None) and
-                (isinstance(resolve, dict)) and
-                (len(resolve) > 0)):
+            if (
+                (resolve is not None)
+                and (isinstance(resolve, dict))
+                and (len(resolve) > 0)
+            ):
                 path = path % resolve
         except:
             raise Exception("Path could not be resolved")
@@ -162,7 +167,7 @@ class DataSource(object):
 
         selected = None
         for p in self.paths:
-            if p['nature'] == nature:
+            if p["nature"] == nature:
                 selected = p
                 break
 
@@ -172,12 +177,14 @@ class DataSource(object):
         if full:
             return selected
 
-        path = selected['path']
+        path = selected["path"]
 
         try:
-            if ((resolve is not None) and
-                (isinstance(resolve, dict)) and
-                (len(resolve) > 0)):
+            if (
+                (resolve is not None)
+                and (isinstance(resolve, dict))
+                and (len(resolve) > 0)
+            ):
                 path = path % resolve
         except:
             raise Exception("Path could not be resolved")
@@ -194,15 +201,15 @@ class DataSource(object):
         """
         Return the names of subsets available
         """
-        return [s['name'] for s in self.subsets]
+        return [s["name"] for s in self.subsets]
 
-    def get_subset_description(self,name):
+    def get_subset_description(self, name):
         """
         Get the description for a subset
         """
         for s in self.subsets:
-            if s['name'] == name:
-                return s.get('description', self.description)
+            if s["name"] == name:
+                return s.get("description", self.description)
 
         raise Exception("Unknown subset: {}".format(name))
 
@@ -211,53 +218,51 @@ class DataSource(object):
         Return the names of subsets available
         """
 
-        if ((not isinstance(spec, dict)) or
-            (len(spec) == 0) or
-            ('filename' not in spec)):
+        if (not isinstance(spec, dict)) or (len(spec) == 0) or ("filename" not in spec):
             raise Exception("invalid subset check input. Expecting dict with filename")
 
         # Does the subset name exist?
         subset = None
         for s in self.subsets:
-            if name == s['name']:
+            if name == s["name"]:
                 subset = s
                 break
         if subset is None:
             return False
 
         #
-        if 'pattern' not in subset:
+        if "pattern" not in subset:
             return False
 
-        filename = spec['filename']
-        pattern = subset['pattern']
+        filename = spec["filename"]
+        pattern = subset["pattern"]
         return re.search(pattern, filename) is not None
 
     def get_path(self, name, full=False, resolve=None):
         return self.get_path_by_name(name, full, resolve)
 
     def validate(self, params):
-        """
-        """
+        """ """
         if not isinstance(params, dict):
             raise Exception("Invalid parameters for dataset")
 
-        for col in ['name', 'paths']:
+        for col in ["name", "paths"]:
             if col not in params:
                 raise Exception("Data specification should have {}".format(col))
 
-        if (not isinstance(params['paths'], list)):
+        if not isinstance(params["paths"], list):
             raise Exception("Paths should be a list of dictionaries")
 
-        for elem in params['paths']:
+        for elem in params["paths"]:
             if not isinstance(elem, dict) or len(elem) == 0:
                 raise Exception("paths element should be non-empty dict")
-            if (('nature' not in elem) or (elem['nature'] not in self.valid_natures)):
-                raise Exception(f"paths element have nature {'|'.join(self.valid_natures)}")
-            if 'path' not in elem:
+            if ("nature" not in elem) or (elem["nature"] not in self.valid_natures):
+                raise Exception(
+                    f"paths element have nature {'|'.join(self.valid_natures)}"
+                )
+            if "path" not in elem:
                 raise Exception("paths element should have a 'path'")
 
-
     def __str__(self):
         return "[{}] {}".format(self._type, self.name)
 
@@ -317,7 +322,7 @@ class Dataset(DataSource):
 
         super().__init__(params, *args, **kwargs)
 
-        self.match = params.get('match',{})
+        self.match = params.get("match", {})
         """
         dict: Generating and matching rules
 
@@ -344,22 +349,25 @@ class Dataset(DataSource):
 
         if fd is None:
             # Text formats..
-            if filename.split(".")[-1].lower() in ['csv', 'tsv']:
+            if filename.split(".")[-1].lower() in ["csv", "tsv"]:
                 fd = open(filename)
             else:
-                fd = open(filename, 'rb')
+                fd = open(filename, "rb")
 
         # Read the file
-        if filename.lower().endswith('.csv'):
+        if filename.lower().endswith(".csv"):
             df = pd.read_csv(fd, nrows=nrows)
-        elif filename.lower().endswith('.csv.gz'):
-            df = pd.read_csv(fd, nrows=nrows, compression='gzip', error_bad_lines=False)
-        elif filename.lower().endswith('.tsv.gz'):
-            df = pd.read_csv(fd, nrows=nrows, sep='\t', compression='gzip',
-                             error_bad_lines=False)
-        elif filename.lower().endswith('.tsv'):
-            df = pd.read_csv(fd, nrows=nrows, sep='\t')
-        elif filename.lower().endswith('.pq') or filename.lower().endswith(".pq.sample"):
+        elif filename.lower().endswith(".csv.gz"):
+            df = pd.read_csv(fd, nrows=nrows, compression="gzip", error_bad_lines=False)
+        elif filename.lower().endswith(".tsv.gz"):
+            df = pd.read_csv(
+                fd, nrows=nrows, sep="\t", compression="gzip", error_bad_lines=False
+            )
+        elif filename.lower().endswith(".tsv"):
+            df = pd.read_csv(fd, nrows=nrows, sep="\t")
+        elif filename.lower().endswith(".pq") or filename.lower().endswith(
+            ".pq.sample"
+        ):
             df = pd.read_parquet(fd)
             if nrows is not None:
                 df = df.head(nrows)
@@ -374,7 +382,7 @@ class Dataset(DataSource):
         Read the path with the name 'local'. Available for
         backward compatability
         """
-        return self.get_path('local')
+        return self.get_path("local")
 
     @property
     def test(self):
@@ -382,7 +390,7 @@ class Dataset(DataSource):
         Read the path with the name 'test'. Available for
         backward compatability
         """
-        return self.get_path('test')
+        return self.get_path("test")
 
     @property
     def backup(self):
@@ -390,11 +398,11 @@ class Dataset(DataSource):
         Read the path with the name 'backup'. Available for
         backward compatability
         """
-        return self.get_path('backup')
+        return self.get_path("backup")
 
     @property
     def valid_natures(self):
-        return ['local', 'remote', 's3', 'gcs']
+        return ["local", "remote", "s3", "gcs"]
 
     def validate(self, params):
         """
@@ -414,25 +422,27 @@ class Dataset(DataSource):
         super().validate(params)
 
         # If match is specified, then do something...
-        if (('match' not in params) or (not isinstance(params['match'], dict))):
+        if ("match" not in params) or (not isinstance(params["match"], dict)):
             raise Exception("Match should be a valid dictionary")
 
-        if 'pattern' not in params['match']:
+        if "pattern" not in params["match"]:
             raise Exception("Matching requirement match.pattern missing")
 
-        pattern = params['match']['pattern']
+        pattern = params["match"]["pattern"]
         if not isinstance(pattern, str) and not callable(pattern):
             raise Exception("Invalid pattern specification")
 
-        generate = params['match'].get('generate', 'generate_datetime_daily')
+        generate = params["match"].get("generate", "generate_datetime_daily")
 
-        if (((isinstance(generate, str)) and (not hasattr(self, generate))) and
-            (not callable(generate))):
+        if ((isinstance(generate, str)) and (not hasattr(self, generate))) and (
+            not callable(generate)
+        ):
             raise Exception("Matching requirement {} missing/invalid".format(generate))
 
-        compare = params['match'].get('compare', 'compare_datetime_pattern_match_range')
-        if (((isinstance(compare, str)) and (not hasattr(self, compare))) and
-            (not callable(compare))):
+        compare = params["match"].get("compare", "compare_datetime_pattern_match_range")
+        if ((isinstance(compare, str)) and (not hasattr(self, compare))) and (
+            not callable(compare)
+        ):
             raise Exception("Matching requirement {} missing/invalid".format(compare))
 
     def generate(self, start, end=None):
@@ -448,9 +458,9 @@ class Dataset(DataSource):
            list (dict): List of dictionaries (name, timestamp)
 
         """
-        if ((not isinstance(start, (datetime,date))) or
-            ((end is not None) and
-             (not isinstance(end, (datetime, date))))):
+        if (not isinstance(start, (datetime, date))) or (
+            (end is not None) and (not isinstance(end, (datetime, date)))
+        ):
             raise Exception("Start or end is not a datetime/date")
 
         # One day
@@ -461,9 +471,8 @@ class Dataset(DataSource):
         if end < start:
             start, end = end, start
 
-
-        func = self.match['generate']
-        if isinstance(func,str):
+        func = self.match["generate"]
+        if isinstance(func, str):
             func = getattr(self, func)
 
         # Generate all possible names
@@ -482,7 +491,7 @@ class Dataset(DataSource):
            end (datetime): End of range. If None, default = start
         """
 
-        pattern = self.match['pattern']
+        pattern = self.match["pattern"]
 
         names = []
         current = start
@@ -494,10 +503,7 @@ class Dataset(DataSource):
             else:
                 raise Exception("Unsupported pattern type")
 
-            names.append({
-                'timestamp': current.isoformat(),
-                'name': name
-            })
+            names.append({"timestamp": current.isoformat(), "name": name})
             current += timedelta(days=1)
 
         return names
@@ -520,12 +526,11 @@ class Dataset(DataSource):
         names = self.generate(start, end)
 
         # names is a dict {'timestamp': '2021-02-02T13:24:11', 'name': '2021-02-02'}
-        names = [n['name'] for n in names]
+        names = [n["name"] for n in names]
         # Check if the name is in the names or in the range. The
         # latter is required if there are multiple runs in a given day
         #
-        return ((name in names) or
-                ((name >= min(names) and name <= max(names))))
+        return (name in names) or ((name >= min(names) and name <= max(names)))
 
     def match_content(self, fs, localdir, backupdir, dirname):
         """
@@ -549,10 +554,12 @@ class Dataset(DataSource):
             except Exception as e:
                 return False, str(e)
 
-            if filesize != detail['size']:
+            if filesize != detail["size"]:
                 return False, "Filesize mismatch: {}".format(dirname)
 
-            return True, "Matched {} (size: {}M)".format(dirname, round(filesize/1000000, 1))
+            return True, "Matched {} (size: {}M)".format(
+                dirname, round(filesize / 1000000, 1)
+            )
 
         matched = 0
         totalsize = 0
@@ -569,14 +576,15 @@ class Dataset(DataSource):
                 except Exception as e:
                     return False, str(e)
 
-                if filesize != detail['size']:
+                if filesize != detail["size"]:
                     return False, "Filesize mismatch: {}".format(relpath)
 
                 matched += 1
                 totalsize += filesize
 
-        return True, "Matched {} (size: {}M)".format(matched, round(totalsize/1000000, 1))
-
+        return True, "Matched {} (size: {}M)".format(
+            matched, round(totalsize / 1000000, 1)
+        )
 
     def compare(self, name, start, end=None):
         """
@@ -589,8 +597,9 @@ class Dataset(DataSource):
            end (datetime): End of range. If None, default = start
 
         """
-        if ((not isinstance(start, datetime))  or
-            ((end is not None) and (not isinstance(end, datetime)))):
+        if (not isinstance(start, datetime)) or (
+            (end is not None) and (not isinstance(end, datetime))
+        ):
             raise Exception("Generate names in the range")
 
         # One day
@@ -601,34 +610,33 @@ class Dataset(DataSource):
         if end < start:
             start, end = end, start
 
-        func = params['match'].get('compare', 'compare_datetime_pattern_match_range')
-        if isinstance(func,str):
+        func = params["match"].get("compare", "compare_datetime_pattern_match_range")
+        if isinstance(func, str):
             func = getattr(self, func)
 
         # Generate all possible names
         return func(name, start, end)
 
-
-    def listdir(self, name,
-                fshandle=None,
-                resolve=None,
-                detail=False):
+    def listdir(self, name, fshandle=None, resolve=None, detail=False):
         """
         List the dataset directory
         """
 
         selected = None
         for p in self.paths:
-            if p['name'] == name:
+            if p["name"] == name:
                 selected = p
                 break
 
         if selected is None:
             raise Exception("Unknown name of path: {}".format(name))
 
-        if ((selected['nature'] in ['s3', 'gcs']) and
-            ((fshandle is None) or (not isinstance(fshandle, S3FileSystem)))):
-            raise Exception("Filesystem handle should be provided when nature is s3|gcs")
+        if (selected["nature"] in ["s3", "gcs"]) and (
+            (fshandle is None) or (not isinstance(fshandle, S3FileSystem))
+        ):
+            raise Exception(
+                "Filesystem handle should be provided when nature is s3|gcs"
+            )
 
         if not isinstance(detail, bool):
             raise Exception("detail is a boolean field")
@@ -636,9 +644,11 @@ class Dataset(DataSource):
         path = self.paths[name]
 
         try:
-            if ((resolve is not None) and
-                (isinstance(resolve, dict)) and
-                (len(resolve) > 0)):
+            if (
+                (resolve is not None)
+                and (isinstance(resolve, dict))
+                and (len(resolve) > 0)
+            ):
                 path = path % resolve
         except:
             raise Exception("Path could not be resolved")
@@ -652,15 +662,16 @@ class Dataset(DataSource):
 
         return results
 
-    def read_data(self,
-                  start_date,
-                  end_date,
-                  filename,
-                  readfunc,
-                  errors=True,
-                  name="default",
-                  params={},
-                  resolve={}
+    def read_data(
+        self,
+        start_date,
+        end_date,
+        filename,
+        readfunc,
+        errors=True,
+        name="default",
+        params={},
+        resolve={},
     ):
         """
         Read a single dataset
@@ -669,14 +680,14 @@ class Dataset(DataSource):
              start_date (str): Starting date to scan
              end_date (str): Ending date of the scane
              filename (str): Filename within each day's directory
-             readfunc (method): Callback 
+             readfunc (method): Callback
              errors (bool): What to do on failure. True = bailout
              name (str): Which subset to look at?
              params (dict): Params to the readfunc
              resolve (dict): Parameters for resolution of path
 
         Returns:
-             dataframe: Data read 
+             dataframe: Data read
              dict: Optional metadata on the read
 
         """
@@ -688,44 +699,43 @@ class Dataset(DataSource):
         # Get the path template...
         path = self.get_path_by_name(name, resolve)
 
-        metadata = {
-            'start': datetime.now(),
-            'files': []
-        }
+        metadata = {"start": datetime.now(), "files": []}
         # Now compose all the dataframes...
         _dfs = []
         for dt in dates:
-            fullpath = os.path.join(path, dt['name'], filename)
+            fullpath = os.path.join(path, dt["name"], filename)
             try:
-                metadata['files'].append(fullpath)
-                df = readfunc(fullpath, params, date=dt['name'])
-                if ((df is None) or (len(df) == 0)):
+                metadata["files"].append(fullpath)
+                df = readfunc(fullpath, params, date=dt["name"])
+                if (df is None) or (len(df) == 0):
                     continue
             except:
                 if errors:
                     raise
             _dfs.append(df)
 
+        if len(_dfs) == 0:
+            raise NoDataFound()
+
         df = pd.concat(_dfs, ignore_index=True)
         del _dfs
 
-        metadata['end'] = datetime.now()
+        metadata["end"] = datetime.now()
 
         return df, metadata
 
 
 class DatabaseTable(DataSource):
-
     def __init__(self, params, *args, **kwargs):
         super().__init__(params, *args, **kwargs)
         self._type = "DBTable"
 
-
     @property
     def valid_natures(self):
-        return ['db']
+        return ["db"]
 
-class Arg():
+
+class Arg:
     """
     Dataset command argument
     """
@@ -743,17 +753,17 @@ class Arg():
     def validate(self, datasource, value):
         raise Exception("Not implemented")
 
-    def resolve(self, datasource, value,resolve=None):
+    def resolve(self, datasource, value, resolve=None):
         raise Exception("Not implemented")
 
+
 class DateArg(Arg):
     """
     Datetime
     """
+
     def __init__(self, name, *args, **kwargs):
-        super().__init__(name,
-                         "Date within past 1 year",
-                         *args, **kwargs)
+        super().__init__(name, "Date within past 1 year", *args, **kwargs)
 
     def find(self, datasource):
         today = datetime.now().date()
@@ -774,21 +784,19 @@ class DateArg(Arg):
         dt = dateparser.parse(value)
         return dt
 
-class DatasetPathNatureArg(Arg):
 
+class DatasetPathNatureArg(Arg):
     def __init__(self, name, nature, *args, **kwargs):
 
-        super().__init__(name,
-                         "Path of type: {}".format(nature),
-                         *args, **kwargs)
+        super().__init__(name, "Path of type: {}".format(nature), *args, **kwargs)
         self.nature = nature
 
     def find(self, datasource):
 
         valid = []
         for p in datasource.paths:
-            if (p['nature'] == self.nature):
-                valid.append(p['name'])
+            if p["nature"] == self.nature:
+                valid.append(p["name"])
 
         return valid
 
@@ -798,69 +806,69 @@ class DatasetPathNatureArg(Arg):
             return False
 
         for p in datasource.paths:
-            if ((p['name'] == value) and
-                (p['nature'] == self.nature)):
+            if (p["name"] == value) and (p["nature"] == self.nature):
                 return True
 
         return False
 
-    def resolve(self, datasource, value,resolve=None):
+    def resolve(self, datasource, value, resolve=None):
 
         if not isinstance(datasource, Dataset):
             raise Exception("Invalid datasource. Expected dataset")
 
-        return datasource.get_path_by_name(value,resolve=resolve)
+        return datasource.get_path_by_name(value, resolve=resolve)
+
 
 def get_commands():
 
-    src_s3    = DatasetPathNatureArg("src", "s3")
-    dst_s3    = DatasetPathNatureArg("dst", "s3")
+    src_s3 = DatasetPathNatureArg("src", "s3")
+    dst_s3 = DatasetPathNatureArg("dst", "s3")
     src_local = DatasetPathNatureArg("src", "local")
     dst_local = DatasetPathNatureArg("dst", "local")
-    start     = DateArg("start")
-    end       = DateArg("end")
+    start = DateArg("start")
+    end = DateArg("end")
 
     commands = [
         {
-            "source_type": ['File'],
+            "source_type": ["File"],
             "args": [src_s3, dst_local, start, end],
             "name": "s3-to-local",
-            'description': "Download to local data directory",
+            "description": "Download to local data directory",
             "command": "aws s3 sync s3://%(source)s/%(subdir)s%(slash)s %(destination)s/%(subdir)s%(slash)s",
         },
         {
-            "source_type": ['File'],
+            "source_type": ["File"],
             "args": [src_local, dst_s3, start, end],
             "name": "local-to-s3",
-            'description': "Upload from local data directory to s3",
+            "description": "Upload from local data directory to s3",
             "command": "aws s3 sync %(source)s/%(subdir)s%(slash)s s3://%(destination)s/%(subdir)s%(slash)s",
         },
         {
-            "source_type": ['File'],
+            "source_type": ["File"],
             "args": [src_s3, dst_s3, start, end],
             "name": "s3-to-s3",
-            'description': "Sync from one s3 path to another s3 path",
+            "description": "Sync from one s3 path to another s3 path",
             "command": "aws s3 sync s3://%(source)s/%(subdir)s%(slash)s s3://%(destination)s/%(subdir)s%(slash)s",
         },
         {
-            "source_type": ['File'],
+            "source_type": ["File"],
             "args": [src_local, dst_local, start, end],
             "name": "remote-to-local",
-            'description': "Copy from remote local to test",
-            "command" : "scp -C -r ubuntu@%(node)s:%(source)s %(destination)s/%(subdir)s"
-
+            "description": "Copy from remote local to test",
+            "command": "scp -C -r ubuntu@%(node)s:%(source)s %(destination)s/%(subdir)s",
         },
         {
-            "source_type": ['File'],
+            "source_type": ["File"],
             "args": [src_local, dst_local, start, end],
             "name": "local-to-remote",
-            'description': "Copy from test to remote local path",
-            "command" : "scp -C -r %(destination)s/%(subdir)s ubuntu@%(node)s:%(source)s"
+            "description": "Copy from test to remote local path",
+            "command": "scp -C -r %(destination)s/%(subdir)s ubuntu@%(node)s:%(source)s",
         },
     ]
 
     return commands
 
+
 class DatasetRegistry(object):
     """
     Registry for datasets.
@@ -872,6 +880,7 @@ class DatasetRegistry(object):
     scripting.
 
     """
+
     def __init__(self, *args, **kwargs):
         """
 
@@ -881,21 +890,21 @@ class DatasetRegistry(object):
             resolve (dict): Path resolution dictionary
 
         """
-        self.commands = kwargs.get('commands', get_commands())
-        self.resolve = kwargs.get('resolve',{})
+        self.commands = kwargs.get("commands", get_commands())
+        self.resolve = kwargs.get("resolve", {})
         self.datasets = []
         self.validate()
 
     def validate(self):
         for c in self.commands:
             assert isinstance(c, dict)
-            assert (('source_type' in c) and (isinstance(c['source_type'],list)))
-            assert 'args' in c
-            assert isinstance(c['args'], list)
-            assert 'name' in c
-            assert 'command' in c
-            assert isinstance(c['command'], str)
-            assert len(c['command']) > 0
+            assert ("source_type" in c) and (isinstance(c["source_type"], list))
+            assert "args" in c
+            assert isinstance(c["args"], list)
+            assert "name" in c
+            assert "command" in c
+            assert isinstance(c["command"], str)
+            assert len(c["command"]) > 0
 
     def get_command(self, name, source_type="File"):
         """
@@ -905,13 +914,13 @@ class DatasetRegistry(object):
             name (str): Name of the command template
         """
         for c in self.commands:
-            if source_type not in c['source_type']:
+            if source_type not in c["source_type"]:
                 continue
-            if c['name'] == name:
+            if c["name"] == name:
                 return c
         raise Exception("Missing command: {}".format(name))
 
-    def get_commands(self, source_type='File'):
+    def get_commands(self, source_type="File"):
         """
         Get the command specifications as a list of dicts
 
@@ -920,7 +929,7 @@ class DatasetRegistry(object):
             return self.commands
 
         # If specified, then apply that filter..
-        return [c for c in self.commands if source_type in c['source_type']]
+        return [c for c in self.commands if source_type in c["source_type"]]
 
     def add_datasets(self, items):
         """
